# -*- coding: utf-8 -*-

"""
    This module provides a registry for all hooks
"""

from singleton import singleton

from . import utils
from .exceptions import HookError


@singleton()
class HookRegistry(object):
    """
        Represents an object with all registered hooks
    """
    def __init__(self):
        self._hooks = {}
        self.reset()
        self.build_hooks()

    @property
    def hooks(self):
        """
            Returns all registered hooks
        """
        return self._hooks

    class Hook(object):
        """
            Represents a hook object

            This object is needed to provide decorators like:
            * @before.all
            * @before.each_feature
        """
        def __init__(self, when):
            self._when = when

        @classmethod
        def build_decorator(cls, what):
            """
                Builds the hook decorator
            """
            def _decorator(self, func):
                """
                    Actual hook decorator
                """
                HookRegistry().register(self._when, what, func)  # pylint: disable=protected-access
                return func
            _decorator.__name__ = _decorator.fn_name = what
            setattr(cls, what, _decorator)

    def build_hooks(self):
        """
            Builds all hooks
        """
        for hook in self._hooks.keys():
            self.Hook.build_decorator(hook)

    def register(self, when, what, func):
        """
            Registers a function as a hook
        """
        self._hooks[what][when].append(func)

    def reset(self):
        """
            Resets all registerd hooks
        """
        self._hooks = {
            "all": {"before": [], "after": []},
            "each_feature": {"before": [], "after": []},
            "each_scenario": {"before": [], "after": []},
            "each_step": {"before": [], "after": []},
        }

    def call(self, when, what, *args, **kwargs):
        """
            Calls a registered hook
        """
        for hook in self._hooks[what][when]:
            try:
                hook(*args, **kwargs)
            except Exception as e:
                raise HookError(hook, utils.Failure(e))
        return None

HookRegistry()
before = HookRegistry.Hook("before")  # pylint: disable=invalid-name
after = HookRegistry.Hook("after")  # pylint: disable=invalid-name



# -*- coding: utf-8 -*-

import threading

from tests.base import *

from radish.hookregistry import HookRegistry, before, after


class HookRegistryTestCase(RadishTestCase):
    """
        Tests for the HookRegistry class
    """
    def test_initializing_hook_registry(self):
        """
            Test the initialization of the hook registry
        """
        empty_hook = {"before": [], "after": []}

        hookregistry = HookRegistry()
        hookregistry.hooks.should.have.length_of(4)
        hookregistry.hooks["all"].should.be.equal(empty_hook)
        hookregistry.hooks["each_feature"].should.be.equal(empty_hook)
        hookregistry.hooks["each_scenario"].should.be.equal(empty_hook)
        hookregistry.hooks["each_step"].should.be.equal(empty_hook)

        HookRegistry.Hook.should.have.property("all")
        HookRegistry.Hook.should.have.property("each_feature")
        HookRegistry.Hook.should.have.property("each_scenario")
        HookRegistry.Hook.should.have.property("each_step")

    def test_register_as_hook(self):
        """
            Test registering function as hook
        """
        empty_hook = {"before": [], "after": []}

        hookregistry = HookRegistry()
        hookregistry.hooks.should.have.length_of(4)
        hookregistry.hooks["all"].should.be.equal(empty_hook)
        hookregistry.hooks["each_feature"].should.be.equal(empty_hook)
        hookregistry.hooks["each_scenario"].should.be.equal(empty_hook)
        hookregistry.hooks["each_step"].should.be.equal(empty_hook)

        hookregistry.hooks["all"]["before"].should.have.length_of(0)
        hookregistry.register("before", "all", "some_func")
        hookregistry.hooks["all"]["before"].should.have.length_of(1)
        hookregistry.hooks["all"]["after"].should.have.length_of(0)

        hookregistry.hooks["each_feature"]["before"].should.have.length_of(0)
        hookregistry.register("before", "each_feature", "some_func")
        hookregistry.hooks["each_feature"]["before"].should.have.length_of(1)
        hookregistry.hooks["each_feature"]["after"].should.have.length_of(0)

        hookregistry.hooks["each_scenario"]["before"].should.have.length_of(0)
        hookregistry.register("before", "each_scenario", "some_func")
        hookregistry.hooks["each_scenario"]["before"].should.have.length_of(1)
        hookregistry.hooks["each_scenario"]["after"].should.have.length_of(0)

        hookregistry.hooks["each_step"]["before"].should.have.length_of(0)
        hookregistry.register("before", "each_step", "some_func")
        hookregistry.hooks["each_step"]["before"].should.have.length_of(1)
        hookregistry.hooks["each_step"]["after"].should.have.length_of(0)

    def test_decorating_with_hook(self):
        """
            Test decorating function as hook
        """
        hookregistry = HookRegistry()

        def some_func():
            pass

        hookregistry.hooks["each_feature"]["before"].should.have.length_of(0)
        before.each_feature(some_func)
        hookregistry.hooks["each_feature"]["before"].should.have.length_of(1)
        hookregistry.hooks["each_feature"]["before"][0].should.be.equal(some_func)

        hookregistry.hooks["each_step"]["after"].should.have.length_of(0)
        after.each_step(some_func)
        hookregistry.hooks["each_step"]["after"].should.have.length_of(1)
        hookregistry.hooks["each_step"]["after"][0].should.be.equal(some_func)

    def test_calling_registered_hooks(self):
        """
            Test calling registered hooks
        """
        hookregistry = HookRegistry()

        data = threading.local()
        data.called_hooked_a = False
        data.called_hooked_b = False
        data.called_hooked_c = False

        def hooked_a():
            data.called_hooked_a = True

        def hooked_b():
            data.called_hooked_b = True

        def hooked_c():
            data.called_hooked_c = True

        before.each_feature(hooked_a)
        after.each_step(hooked_b)
        before.each_feature(hooked_c)

        hookregistry.call("before", "each_feature")
        data.called_hooked_a.should.be.true
        data.called_hooked_b.should.be.false
        data.called_hooked_c.should.be.true

        hookregistry.call("after", "each_step")
        data.called_hooked_a.should.be.true
        data.called_hooked_b.should.be.true
        data.called_hooked_c.should.be.true

    def test_calling_registered_hooks_with_arguments(self):
        """
            Test calling registered hooks with arguments
        """
        hookregistry = HookRegistry()

        data = threading.local()
        data.hooked_a_feature = None
        data.hooked_b_step = None
        data.hooked_c_feature = None

        def hooked_a(feature):
            data.hooked_a_feature = feature

        def hooked_b(step):
            data.hooked_b_step = step

        def hooked_c(feature):
            data.hooked_c_feature = feature

        before.each_feature(hooked_a)
        after.each_step(hooked_b)
        before.each_feature(hooked_c)

        hookregistry.call("before", "each_feature", "this is a feature")
        data.hooked_a_feature = "this is a feature"
        data.hooked_b_step = None
        data.hooked_c_feature = "this is a feature"

        hookregistry.call("after", "each_step", "this is a step")
        data.hooked_a_feature = "this is a feature"
        data.hooked_b_step = "this is a step"
        data.hooked_c_feature = "this is a feature"



# -*- coding: utf-8 -*-

import re
import threading

from tests.base import *

from radish.stepmodel import Step
from radish.stepregistry import StepRegistry, step, given, then
from radish.exceptions import RadishError, StepRegexError


class StepTestCase(RadishTestCase):
    """
        Tests for the step functionality
    """
    def test_registering_steps_with_decorator(self):
        """
            Test registering steps with decorator
        """
        registry = StepRegistry()
        registry.steps.should.have.length_of(0)

        def step_a():
            pass

        def step_b():
            pass

        step("abc")(step_a)
        registry.steps.should.have.length_of(1)
        registry.steps["abc"].should.be.equal(step_a)

        step("def")(step_b)
        registry.steps.should.have.length_of(2)
        registry.steps["def"].should.be.equal(step_b)

    def test_registering_steps_with_gherkin_decorators(self):
        """
            Test registering steps with gherkin specific decorators
        """
        def step_given():
            pass

        def step_when():
            pass

        def step_then():
            pass

        registry = StepRegistry()
        registry.steps.should.be.empty

        given(r"I have the number {:g}")(step_given)
        registry.steps.should.have.length_of(1)
        registry.steps[r"Given I have the number {:g}"] = step_given

        # cannot test the when function because sure overwrites it!
        # stupid stuff
        # step_when(r"I add \d+ to my number")(step_when)
        # registry.steps.should.have.length_of(2)
        # registry.steps[r"When I add \d+ to my number"] = step_when

        then(re.compile(r"I expect my number to be \d+"))(step_then)
        registry.steps.should.have.length_of(2)

    def test_run_step_passed(self):
        """
            Test running a passing step
        """
        data = threading.local()
        data.step_was_run = False

        def step_passed(step):
            data.step_was_run = True

        step = Step(1, "I call a passing step", "somefile.feature", 3, None, True)
        step.definition_func = step_passed
        argument_match_mock = Mock()
        argument_match_mock.evaluate.return_value = (re.search(step.sentence, step.sentence).groups(), {})
        step.argument_match = argument_match_mock

        step.state.should.be.equal(Step.State.UNTESTED)
        step.run.when.called_with().should.return_value(Step.State.PASSED)
        data.step_was_run.should.be.true

    def test_run_step_with_arguments_passed(self):
        """
            Test running a passing step with arguments
        """
        data = threading.local()
        data.step_was_run = False
        data.number = None
        data.string = None

        def step_passed(step, number, string):
            data.step_was_run = True
            data.number = int(number)
            data.string = string

        step = Step(1, "I call a passing step with number argument 42 and string argument 'Tschau'", "somefile.feature", 3, None, True)
        step.definition_func = step_passed
        argument_match_mock = Mock()
        argument_match_mock.evaluate.return_value = (
            re.search(r"I call a passing step with number argument (\d+) and string argument '(.*?)'", step.sentence).groups(), {})
        step.argument_match = argument_match_mock

        step.state.should.be.equal(Step.State.UNTESTED)
        step.run.when.called_with().should.return_value(Step.State.PASSED)
        data.step_was_run.should.be.true
        data.number.should.be.equal(42)
        data.string.should.be.equal("Tschau")

    def test_run_step_with_keyword_arguments_passed(self):
        """
            Test running a passing step with keyword arguments
        """
        data = threading.local()
        data.step_was_run = False
        data.number = None
        data.string = None

        def step_passed(step, number, string):
            data.step_was_run = True
            data.number = int(number)
            data.string = string

        step = Step(1, "I call a passing step with string argument 'Tschau' and number argument 42", "somefile.feature", 3, None, True)
        step.definition_func = step_passed
        match = re.search("I call a passing step with string argument '(?P<string>.*?)' and number argument (?P<number>\d+)", step.sentence)
        argument_match_mock = Mock()
        argument_match_mock.evaluate.return_value = (match.groups(), match.groupdict())
        step.argument_match = argument_match_mock

        step.state.should.be.equal(Step.State.UNTESTED)
        step.run.when.called_with().should.return_value(Step.State.PASSED)
        data.step_was_run.should.be.true
        data.number.should.be.equal(42)
        data.string.should.be.equal("Tschau")

    def test_run_step_failed(self):
        """
            Test running a failing step
        """
        data = threading.local()
        data.step_was_run = False

        def step_failed(step):
            data.step_was_run = True
            assert False, "This step fails by design"

        step = Step(1, "I call a failing step", "somefile.feature", 3, None, True)
        step.definition_func = step_failed
        argument_match_mock = Mock()
        argument_match_mock.evaluate.return_value = (re.search(step.sentence, step.sentence).groups(), {})
        step.argument_match = argument_match_mock

        step.state.should.be.equal(Step.State.UNTESTED)
        step.run.when.called_with().should.return_value(Step.State.FAILED)
        step.failure.shouldnt.be.none
        step.failure.reason.should.be.equal("This step fails by design")
        data.step_was_run.should.be.true

    def test_run_a_step_without_definition(self):
        """
            Test running a step without assining a definition
        """
        step = Step(1, "I call an invalid step", "somefile.feature", 3, None, True)
        step.run.when.called_with().should.throw(RadishError, "The step 'I call an invalid step' does not have a step definition")



# -*- coding: utf-8 -*-

"""
    This module provides some default ArgumentExpressions
"""

from radish.argexpregistry import arg_expr


@arg_expr("MathExpression", r"[0-9 +\-*/%.e]+")
def arg_expr_mathexpression(text):
    """
        Argument Expression which expects a valid math expression

        :param str text: the text which was matched as math expression

        :returns: calculated float number from the math expression
        :rtype: float
    """
    return float(eval(text))



# -*- coding: utf-8 -*-

from tests.base import *

from radish.stepregistry import StepRegistry
from radish.exceptions import SameStepError


class StepRegistryTestCase(RadishTestCase):
    """
        Tests for the StepRegistry class
    """
    def test_registering_steps(self):
        """
            Test registering multiple steps
        """
        registry = StepRegistry()
        registry.steps.should.be.empty

        def step_a():
            pass

        def step_b():
            pass

        registry.register("abc", step_a)
        registry.steps.should.have.length_of(1)
        registry.steps["abc"].should.be.equal(step_a)

        registry.register("def", step_b)
        registry.steps.should.have.length_of(2)
        registry.steps["def"].should.be.equal(step_b)

    def test_registering_same_step(self):
        """
            Test registering step with same regex
        """
        registry = StepRegistry()
        registry.steps.should.be.empty

        def step_a():
            pass

        def step_b():
            pass

        registry.register("abc", step_a)
        registry.steps.should.have.length_of(1)
        registry.steps["abc"].should.be.equal(step_a)

        registry.register.when.called_with("abc", step_b).should.throw(SameStepError, "Cannot register step step_b with regex 'abc' because it is already used by step step_a")

        registry.steps.should.have.length_of(1)
        registry.steps["abc"].should.be.equal(step_a)

    def test_registering_object(self):
        """
            Test registering object as steps
        """
        registry = StepRegistry()
        registry.steps.should.be.empty

        class MySteps(object):
            ignore = ["no_step_method"]

            def some_step(step):
                """When I call some step"""

            def some_other_step(step):
                """
                    Then I expect some behaviour
                """

            def no_step_method(data):
                """
                    This is not a step method
                """

        steps_object = MySteps()
        registry.register_object(steps_object)

        registry.steps.should.have.length_of(2)
        registry.steps["When I call some step"].should.be.equal(steps_object.some_step)
        registry.steps["Then I expect some behaviour"].should.be.equal(steps_object.some_other_step)



# -*- coding: utf-8 -*-

from tests.base import *

from tempfile import NamedTemporaryFile
import tagexpressions

from radish.parser import FeatureParser
from radish.feature import Feature
from radish.scenario import Scenario
from radish.scenariooutline import ScenarioOutline
from radish.scenarioloop import ScenarioLoop
from radish.exceptions import RadishError, LanguageNotSupportedError, FeatureFileSyntaxError


class ParserTestCase(RadishTestCase):
    """
        Tests for the parser class
    """
    def test_language_loading(self):
        """
            Test loading of a specific language
        """
        core = Mock()
        en_feature_parser = FeatureParser(core, "/", 1, language="en")
        en_feature_parser.keywords.feature.should.be.equal("Feature")
        en_feature_parser.keywords.scenario.should.be.equal("Scenario")
        en_feature_parser.keywords.scenario_outline.should.be.equal("Scenario Outline")
        en_feature_parser.keywords.examples.should.be.equal("Examples")

        de_feature_parser = FeatureParser(core, "/", 1, language="de")
        de_feature_parser.keywords.scenario.should.be.equal("Szenario")
        de_feature_parser.keywords.scenario_outline.should.be.equal("Szenario Auslagerung")
        de_feature_parser.keywords.examples.should.be.equal("Beispiele")

        FeatureParser.when.called_with(core, "/", 1, language="foo").should.throw(LanguageNotSupportedError)

    def test_parse_unexisting_featurefile(self):
        """
            Test parsing of an unexisting featurefile
        """
        core = Mock()
        FeatureParser.when.called_with(core, "nonexisting.feature", 1).should.throw(OSError, "Feature file at 'nonexisting.feature' does not exist")

    def test_parse_empty_featurefile(self):
        """
            Test parsing of an empty feature file
        """
        core = Mock()
        with NamedTemporaryFile("w+") as featurefile:
            parser = FeatureParser(core, featurefile.name, 1)
            parser.parse.when.called_with().should.throw(RadishError, "No Feature found in file {0}".format(featurefile.name))

    def test_parse_empty_feature(self):
        """
            Test parsing of an empty feature
        """
        feature = """Feature: some empty feature"""

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            parser.parse()

            parser.feature.sentence.should.be.equal("some empty feature")
            parser.feature.id.should.be.equal(1)
            parser.feature.path.should.be.equal(featurefile.name)
            parser.feature.line.should.be.equal(1)
            parser.feature.scenarios.should.be.empty

    def test_parse_empty_feature_with_description(self):
        """
            Test parsing of an empty feature with desription
        """
        feature = """Feature: some empty feature
    In order to support cool software
    I do fancy BDD testing with radish"""

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            parser.parse()

            parser.feature.sentence.should.be.equal("some empty feature")
            parser.feature.id.should.be.equal(1)
            parser.feature.path.should.be.equal(featurefile.name)
            parser.feature.line.should.be.equal(1)
            parser.feature.scenarios.should.be.empty
            parser.feature.description.should.have.length_of(2)
            parser.feature.description[0].should.be.equal("In order to support cool software")
            parser.feature.description[1].should.be.equal("I do fancy BDD testing with radish")

    def test_parse_featurefile_with_multiple_features(self):
        """
            Test parsing a feature file with multiple features
        """
        feature = """Feature: some empty feature
Feature: another empty feature"""

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            parser.parse.when.called_with().should.throw(RadishError, "radish supports only one Feature per feature file")

    def test_parse_feature_with_empty_scenario(self):
        """
            Test parsing of a feature with one empty scenario
        """
        feature = """Feature: some feature
    Scenario: some empty scenario"""

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            parser.parse()

            parser.feature.id.should.be.equal(1)
            parser.feature.sentence.should.be.equal("some feature")
            parser.feature.scenarios.should.have.length_of(1)
            parser.feature.scenarios[0].id.should.be.equal(1)
            parser.feature.scenarios[0].sentence.should.be.equal("some empty scenario")
            parser.feature.scenarios[0].path.should.be.equal(featurefile.name)
            parser.feature.scenarios[0].line.should.be.equal(2)
            parser.feature.scenarios[0].steps.should.be.empty

    def test_parse_feature_with_one_scenario_and_steps(self):
        """
            Test parsing of a feautre with one scenario and multiple steps
        """
        feature = """Feature: some feature
    Scenario: some fancy scenario
        Given I have the number 5
        When I add 2 to my number
        Then I expect my number to be 7 """

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            parser.parse()

            parser.feature.id.should.be.equal(1)
            parser.feature.sentence.should.be.equal("some feature")
            parser.feature.scenarios.should.have.length_of(1)
            parser.feature.scenarios.should.have.length_of(1)
            parser.feature.scenarios[0].sentence.should.be.equal("some fancy scenario")

            parser.feature.scenarios[0].steps.should.have.length_of(3)
            parser.feature.scenarios[0].steps[0].id.should.be.equal(1)
            parser.feature.scenarios[0].steps[0].sentence.should.be.equal("Given I have the number 5")
            parser.feature.scenarios[0].steps[0].path.should.be.equal(featurefile.name)
            parser.feature.scenarios[0].steps[0].line.should.be.equal(3)
            parser.feature.scenarios[0].steps[1].id.should.be.equal(2)
            parser.feature.scenarios[0].steps[1].sentence.should.be.equal("When I add 2 to my number")
            parser.feature.scenarios[0].steps[1].path.should.be.equal(featurefile.name)
            parser.feature.scenarios[0].steps[1].line.should.be.equal(4)
            parser.feature.scenarios[0].steps[2].id.should.be.equal(3)
            parser.feature.scenarios[0].steps[2].sentence.should.be.equal("Then I expect my number to be 7")
            parser.feature.scenarios[0].steps[2].path.should.be.equal(featurefile.name)
            parser.feature.scenarios[0].steps[2].line.should.be.equal(5)

    def test_parse_feature_with_multiple_scenarios(self):
        """
            Test parsing of a feature with multiple scenarios and steps
        """
        feature = """Feature: some feature
    Scenario: some fancy scenario
        Given I have the number 5
        When I add 2 to my number
        Then I expect my number to be 7

    Scenario: some other fancy scenario
        Given I have the number 50
        When I add 20 to my number
        Then I expect my number to be 70"""

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            parser.parse()

            parser.feature.id.should.be.equal(1)
            parser.feature.sentence.should.be.equal("some feature")
            parser.feature.scenarios.should.have.length_of(2)

            parser.feature.scenarios[0].id.should.be.equal(1)
            parser.feature.scenarios[0].sentence.should.be.equal("some fancy scenario")
            parser.feature.scenarios[0].steps.should.have.length_of(3)
            parser.feature.scenarios[0].steps[0].id.should.be.equal(1)
            parser.feature.scenarios[0].steps[0].sentence.should.be.equal("Given I have the number 5")
            parser.feature.scenarios[0].steps[0].path.should.be.equal(featurefile.name)
            parser.feature.scenarios[0].steps[0].line.should.be.equal(3)
            parser.feature.scenarios[0].steps[1].id.should.be.equal(2)
            parser.feature.scenarios[0].steps[1].sentence.should.be.equal("When I add 2 to my number")
            parser.feature.scenarios[0].steps[1].path.should.be.equal(featurefile.name)
            parser.feature.scenarios[0].steps[1].line.should.be.equal(4)
            parser.feature.scenarios[0].steps[2].id.should.be.equal(3)
            parser.feature.scenarios[0].steps[2].sentence.should.be.equal("Then I expect my number to be 7")
            parser.feature.scenarios[0].steps[2].path.should.be.equal(featurefile.name)
            parser.feature.scenarios[0].steps[2].line.should.be.equal(5)

            parser.feature.scenarios[1].id.should.be.equal(2)
            parser.feature.scenarios[1].sentence.should.be.equal("some other fancy scenario")
            parser.feature.scenarios[1].steps.should.have.length_of(3)
            parser.feature.scenarios[1].steps[0].id.should.be.equal(1)
            parser.feature.scenarios[1].steps[0].sentence.should.be.equal("Given I have the number 50")
            parser.feature.scenarios[1].steps[0].path.should.be.equal(featurefile.name)
            parser.feature.scenarios[1].steps[0].line.should.be.equal(8)
            parser.feature.scenarios[1].steps[1].id.should.be.equal(2)
            parser.feature.scenarios[1].steps[1].sentence.should.be.equal("When I add 20 to my number")
            parser.feature.scenarios[1].steps[1].path.should.be.equal(featurefile.name)
            parser.feature.scenarios[1].steps[1].line.should.be.equal(9)
            parser.feature.scenarios[1].steps[2].id.should.be.equal(3)
            parser.feature.scenarios[1].steps[2].sentence.should.be.equal("Then I expect my number to be 70")
            parser.feature.scenarios[1].steps[2].path.should.be.equal(featurefile.name)
            parser.feature.scenarios[1].steps[2].line.should.be.equal(10)

    def test_feature_with_comments(self):
        """
            Test parsing a feature with comments
        """
        feature = """Feature: some feature
    # this is a comment
    Scenario: some fancy scenario
        Given I have the number 5
        When I add 2 to my number
        # this is another comment
        Then I expect my number to be 7

    Scenario: some other fancy scenario
        Given I have the number 50
        # foobar comment
        When I add 20 to my number
        Then I expect my number to be 70
        # another stupid comment"""

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            parser.parse()

            parser.feature.sentence.should.be.equal("some feature")
            parser.feature.scenarios.should.have.length_of(2)

            parser.feature.scenarios[0].sentence.should.be.equal("some fancy scenario")
            parser.feature.scenarios[0].line.should.be.equal(3)
            parser.feature.scenarios[0].steps.should.have.length_of(3)
            parser.feature.scenarios[0].steps[0].sentence.should.be.equal("Given I have the number 5")
            parser.feature.scenarios[0].steps[0].path.should.be.equal(featurefile.name)
            parser.feature.scenarios[0].steps[0].line.should.be.equal(4)
            parser.feature.scenarios[0].steps[1].sentence.should.be.equal("When I add 2 to my number")
            parser.feature.scenarios[0].steps[1].path.should.be.equal(featurefile.name)
            parser.feature.scenarios[0].steps[1].line.should.be.equal(5)
            parser.feature.scenarios[0].steps[2].sentence.should.be.equal("Then I expect my number to be 7")
            parser.feature.scenarios[0].steps[2].path.should.be.equal(featurefile.name)
            parser.feature.scenarios[0].steps[2].line.should.be.equal(7)

            parser.feature.scenarios[1].sentence.should.be.equal("some other fancy scenario")
            parser.feature.scenarios[1].line.should.be.equal(9)
            parser.feature.scenarios[1].steps.should.have.length_of(3)
            parser.feature.scenarios[1].steps[0].sentence.should.be.equal("Given I have the number 50")
            parser.feature.scenarios[1].steps[0].path.should.be.equal(featurefile.name)
            parser.feature.scenarios[1].steps[0].line.should.be.equal(10)
            parser.feature.scenarios[1].steps[1].sentence.should.be.equal("When I add 20 to my number")
            parser.feature.scenarios[1].steps[1].path.should.be.equal(featurefile.name)
            parser.feature.scenarios[1].steps[1].line.should.be.equal(12)
            parser.feature.scenarios[1].steps[2].sentence.should.be.equal("Then I expect my number to be 70")
            parser.feature.scenarios[1].steps[2].path.should.be.equal(featurefile.name)
            parser.feature.scenarios[1].steps[2].line.should.be.equal(13)

    def test_parse_feature_with_scenario_outline(self):
        """
            Test parsing of a feature file with a scenario outline
        """
        feature = """Feature: some feature
    Scenario Outline: some fancy scenario
        Given I have the number <number>
        When I add <delta> to my number
        Then I expect my number to be <result>

    Examples:
        | number | delta | result |
        | 5      | 2     | 7      |
        | 10     | 3     | 13     |
        | 15     | 6     | 21     |
    """

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            parser.parse()

            parser.feature.id.should.be.equal(1)
            parser.feature.sentence.should.be.equal("some feature")
            parser.feature.scenarios.should.have.length_of(1)

            scenario = parser.feature.scenarios[0]
            scenario.should.be.a(ScenarioOutline)
            scenario.id.should.be.equal(1)
            scenario.sentence.should.be.equal("some fancy scenario")

            scenario.steps.should.have.length_of(3)
            scenario.steps[0].id.should.be.equal(1)
            scenario.steps[0].sentence.should.be.equal("Given I have the number <number>")
            scenario.steps[0].path.should.be.equal(featurefile.name)
            scenario.steps[0].line.should.be.equal(3)
            scenario.steps[1].id.should.be.equal(2)
            scenario.steps[1].sentence.should.be.equal("When I add <delta> to my number")
            scenario.steps[1].path.should.be.equal(featurefile.name)
            scenario.steps[1].line.should.be.equal(4)
            scenario.steps[2].id.should.be.equal(3)
            scenario.steps[2].sentence.should.be.equal("Then I expect my number to be <result>")
            scenario.steps[2].path.should.be.equal(featurefile.name)
            scenario.steps[2].line.should.be.equal(5)

            scenario.examples_header.should.be.equal(["number", "delta", "result"])
            scenario.examples.should.have.length_of(3)
            scenario.examples[0].data.should.be.equal(["5", "2", "7"])
            scenario.examples[1].data.should.be.equal(["10", "3", "13"])
            scenario.examples[2].data.should.be.equal(["15", "6", "21"])

            scenario.scenarios.should.have.length_of(3)
            scenario.scenarios[0].id.should.be.equal(2)
            scenario.scenarios[0].steps.should.have.length_of(3)
            scenario.scenarios[0].steps[0].id.should.be.equal(1)
            scenario.scenarios[0].steps[0].sentence.should.be.equal("Given I have the number 5")
            scenario.scenarios[0].steps[1].id.should.be.equal(2)
            scenario.scenarios[0].steps[1].sentence.should.be.equal("When I add 2 to my number")
            scenario.scenarios[0].steps[2].id.should.be.equal(3)
            scenario.scenarios[0].steps[2].sentence.should.be.equal("Then I expect my number to be 7")

            scenario.scenarios[1].id.should.be.equal(3)
            scenario.scenarios[1].steps.should.have.length_of(3)
            scenario.scenarios[0].steps[0].id.should.be.equal(1)
            scenario.scenarios[1].steps[0].sentence.should.be.equal("Given I have the number 10")
            scenario.scenarios[1].steps[1].id.should.be.equal(2)
            scenario.scenarios[1].steps[1].sentence.should.be.equal("When I add 3 to my number")
            scenario.scenarios[1].steps[2].id.should.be.equal(3)
            scenario.scenarios[1].steps[2].sentence.should.be.equal("Then I expect my number to be 13")

            scenario.scenarios[2].id.should.be.equal(4)
            scenario.scenarios[2].steps.should.have.length_of(3)
            scenario.scenarios[2].steps[0].id.should.be.equal(1)
            scenario.scenarios[2].steps[0].sentence.should.be.equal("Given I have the number 15")
            scenario.scenarios[2].steps[1].id.should.be.equal(2)
            scenario.scenarios[2].steps[1].sentence.should.be.equal("When I add 6 to my number")
            scenario.scenarios[2].steps[2].id.should.be.equal(3)
            scenario.scenarios[2].steps[2].sentence.should.be.equal("Then I expect my number to be 21")

    def test_parse_feature_with_scenario_and_scenario_outline(self):
        """
            Test parsing of a feature file with a scenario and a scenario outline
        """
        feature = """Feature: some feature
    Scenario: some normal scenario
        Given I do some stuff
        When I modify it
        Then I expect something else

    Scenario Outline: some fancy scenario
        Given I have the number <number>
        When I add <delta> to my number
        Then I expect my number to be <result>

    Examples:
        | number | delta | result |
        | 5      | 2     | 7      |
        | 10     | 3     | 13     |
        | 15     | 6     | 21     |

    Scenario: some other normal scenario
        Given I do some other stuff
        When I modify it
        Then I expect something else
    """

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            parser.parse()

            parser.feature.id.should.be.equal(1)
            parser.feature.sentence.should.be.equal("some feature")
            parser.feature.scenarios.should.have.length_of(3)

            parser.feature.scenarios[0].should.be.a(Scenario)
            parser.feature.scenarios[0].id.should.be.equal(1)
            parser.feature.scenarios[0].sentence.should.be.equal("some normal scenario")
            parser.feature.scenarios[0].steps.should.have.length_of(3)
            parser.feature.scenarios[0].steps[0].sentence.should.be.equal("Given I do some stuff")
            parser.feature.scenarios[0].steps[1].sentence.should.be.equal("When I modify it")
            parser.feature.scenarios[0].steps[2].sentence.should.be.equal("Then I expect something else")

            parser.feature.scenarios[1].should.be.a(ScenarioOutline)
            parser.feature.scenarios[1].id.should.be.equal(2)
            parser.feature.scenarios[1].sentence.should.be.equal("some fancy scenario")
            parser.feature.scenarios[1].steps.should.have.length_of(3)
            parser.feature.scenarios[1].steps[0].sentence.should.be.equal("Given I have the number <number>")
            parser.feature.scenarios[1].steps[1].sentence.should.be.equal("When I add <delta> to my number")
            parser.feature.scenarios[1].steps[2].sentence.should.be.equal("Then I expect my number to be <result>")

            parser.feature.scenarios[1].examples_header.should.be.equal(["number", "delta", "result"])
            parser.feature.scenarios[1].examples.should.have.length_of(3)
            parser.feature.scenarios[1].examples[0].data.should.be.equal(["5", "2", "7"])
            parser.feature.scenarios[1].examples[1].data.should.be.equal(["10", "3", "13"])
            parser.feature.scenarios[1].examples[2].data.should.be.equal(["15", "6", "21"])

            parser.feature.scenarios[2].should.be.a(Scenario)
            parser.feature.scenarios[2].id.should.be.equal(6)
            parser.feature.scenarios[2].sentence.should.be.equal("some other normal scenario")
            parser.feature.scenarios[2].steps.should.have.length_of(3)
            parser.feature.scenarios[2].steps[0].sentence.should.be.equal("Given I do some other stuff")
            parser.feature.scenarios[2].steps[1].sentence.should.be.equal("When I modify it")
            parser.feature.scenarios[2].steps[2].sentence.should.be.equal("Then I expect something else")

    def test_parse_feature_with_scenario_and_examples(self):
        """
            Test parsing of a feature with a scenario which has examples
        """
        feature = """Feature: some feature
    Scenario: some fancy scenario
        Given I have the number <number>
        When I add <delta> to my number
        Then I expect my number to be <result>

    Examples:
        | number | delta | result |
        | 5      | 2     | 7      |
        | 10     | 3     | 13     |
        | 15     | 6     | 21     |
    """

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            parser.parse.when.called_with().should.throw(RadishError, "Scenario does not support Examples. Use 'Scenario Outline'")

    def test_parse_steps_with_table(self):
        """
            Test parsing of a feature with a scenario and steps with a table
        """
        feature = """Feature: some feature
    Scenario: some normal scenario
        Given I have the user
            | Bruce     | Wayne      | Batman      |
            | Chuck     | Norris     | PureAwesome |
            | Peter     | Parker     | Spiderman   |
        When I register them in the database
        Then I expect 3 entries in the database
    """

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            parser.parse()

            parser.feature.sentence.should.be.equal("some feature")
            parser.feature.scenarios.should.have.length_of(1)

            parser.feature.scenarios[0].should.be.a(Scenario)
            parser.feature.scenarios[0].sentence.should.be.equal("some normal scenario")
            parser.feature.scenarios[0].steps.should.have.length_of(3)
            parser.feature.scenarios[0].steps[0].sentence.should.be.equal("Given I have the user")
            parser.feature.scenarios[0].steps[0].table.should.have.length_of(3)
            parser.feature.scenarios[0].steps[0].table[0].should.be.equal(["Bruce", "Wayne", "Batman"])
            parser.feature.scenarios[0].steps[0].table[1].should.be.equal(["Chuck", "Norris", "PureAwesome"])
            parser.feature.scenarios[0].steps[0].table[2].should.be.equal(["Peter", "Parker", "Spiderman"])

            parser.feature.scenarios[0].steps[1].sentence.should.be.equal("When I register them in the database")
            parser.feature.scenarios[0].steps[1].table.should.have.length_of(0)

            parser.feature.scenarios[0].steps[2].sentence.should.be.equal("Then I expect 3 entries in the database")
            parser.feature.scenarios[0].steps[2].table.should.have.length_of(0)

    def test_detect_scenario_loop(self):
        """
            Test detecting ScenarioLoop on a given line
        """
        line = "Scenario Loop 10: Some fancy scenario loop"

        with NamedTemporaryFile("w+") as featurefile:
            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            result = parser._detect_scenario_loop(line)

            result.should.be.a(tuple)
            result[0].should.be.equal("Some fancy scenario loop")
            result[1].should.be.equal(10)

            parser._detect_scenario_loop.when.called_with("").should.return_value(None)
            parser._detect_scenario_loop.when.called_with("Scenario: Some fancy scenario").should.return_value(None)
            parser._detect_scenario_loop.when.called_with("Scenario Outline: Some fancy scenario").should.return_value(None)
            parser._detect_scenario_loop.when.called_with("Scenario Loop: Some fancy scenario").should.return_value(None)
            parser._detect_scenario_loop.when.called_with("Scenario Loop 5.5: Some fancy scenario").should.return_value(None)

    def test_parse_feature_with_scenario_loop(self):
        """
            Test parsing of a feature file with a scenario loop
        """
        feature = """Feature: some feature
    Scenario Loop 10: some fancy scenario
        Given I have the number 1
        When I add 2 to my number
        Then I expect my number to be 3
    """

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            parser.parse()

            parser.feature.id.should.be.equal(1)
            parser.feature.sentence.should.be.equal("some feature")
            parser.feature.scenarios.should.have.length_of(1)

            scenario = parser.feature.scenarios[0]
            scenario.should.be.a(ScenarioLoop)
            scenario.id.should.be.equal(1)
            scenario.sentence.should.be.equal("some fancy scenario")

            scenario.steps.should.have.length_of(3)
            scenario.steps[0].id.should.be.equal(1)
            scenario.steps[0].sentence.should.be.equal("Given I have the number 1")
            scenario.steps[0].path.should.be.equal(featurefile.name)
            scenario.steps[0].line.should.be.equal(3)
            scenario.steps[1].id.should.be.equal(2)
            scenario.steps[1].sentence.should.be.equal("When I add 2 to my number")
            scenario.steps[1].path.should.be.equal(featurefile.name)
            scenario.steps[1].line.should.be.equal(4)
            scenario.steps[2].id.should.be.equal(3)
            scenario.steps[2].sentence.should.be.equal("Then I expect my number to be 3")
            scenario.steps[2].path.should.be.equal(featurefile.name)
            scenario.steps[2].line.should.be.equal(5)

            scenario.scenarios.should.have.length_of(10)
            for i in range(10):
                scenario.scenarios[i].id.should.be.equal(i + 2)
                scenario.scenarios[i].steps.should.have.length_of(3)
                scenario.scenarios[i].steps[0].id.should.be.equal(1)
                scenario.scenarios[i].steps[0].sentence.should.be.equal("Given I have the number 1")
                scenario.scenarios[i].steps[1].id.should.be.equal(2)
                scenario.scenarios[i].steps[1].sentence.should.be.equal("When I add 2 to my number")
                scenario.scenarios[i].steps[2].id.should.be.equal(3)
                scenario.scenarios[i].steps[2].sentence.should.be.equal("Then I expect my number to be 3")

    def test_detect_tag(self):
        """
            Test detecting a tag
        """
        feature = """Feature: some feature"""

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            parser._detect_tag.when.called_with("@some_tag").should.return_value(("some_tag", None))
            parser._detect_tag.when.called_with("@some_tag sdfg").should.return_value(("some_tag", None))
            parser._detect_tag.when.called_with("@some_tag_with_arg(args)").should.return_value(("some_tag_with_arg", "args"))
            parser._detect_tag.when.called_with("some_tag").should.return_value(None)
            parser._detect_tag.when.called_with("some_tag sdfg").should.return_value(None)

    def test_parse_feature_with_tag(self):
        """
            Test parsing feature with tag
        """
        feature = """
@some_feature
Feature: some feature
    Scenario Loop 10: some fancy scenario
        Given I have the number 1
        When I add 2 to my number
        Then I expect my number to be 3
    """

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            parser.parse()

            parser.feature.tags.should.have.length_of(1)
            parser.feature.tags[0].name.should.be.equal("some_feature")

    def test_parse_feature_with_multiple_tags(self):
        """
            Test parsing feature with tags
        """
        feature = """
@some_feature
@has_scenario_loop
@add_numbers
Feature: some feature
    Scenario Loop 10: some fancy scenario
        Given I have the number 1
        When I add 2 to my number
        Then I expect my number to be 3
    """

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            parser.parse()

            parser.feature.tags.should.have.length_of(3)
            parser.feature.tags[0].name.should.be.equal("some_feature")
            parser.feature.tags[1].name.should.be.equal("has_scenario_loop")
            parser.feature.tags[2].name.should.be.equal("add_numbers")

    def test_parse_feature_with_scenario_with_tags(self):
        """
            Test parsing feature with scenario and multiple tags
        """
        feature = """
@some_feature
Feature: some feature
    @some_scenario_loop
    Scenario Loop 10: some fancy scenario
        Given I have the number 1
        When I add 2 to my number
        Then I expect my number to be 3

    Scenario: foo
        When I have a normal scenario
        Then I expect nothing special

    @error_case
    @bad_case
    Scenario: bad case
        Given I have the number 1
        When I add 3 to my number
        Then I expect my number not to be 4
    """

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            parser.parse()

            parser.feature.tags.should.have.length_of(1)
            parser.feature.tags[0].name.should.be.equal("some_feature")

            parser.feature.scenarios.should.have.length_of(3)

            parser.feature.scenarios[0].tags.should.have.length_of(1)
            parser.feature.scenarios[0].tags[0].name.should.be.equal("some_scenario_loop")

            parser.feature.scenarios[1].tags.should.be.empty

            parser.feature.scenarios[2].tags.should.have.length_of(2)
            parser.feature.scenarios[2].tags[0].name.should.be.equal("error_case")
            parser.feature.scenarios[2].tags[1].name.should.be.equal("bad_case")

    def test_parse_feature_with_ignored_tag(self):
        """
            Test parsing a feature which is not part of the feature tag expression
        """
        feature = """
@some_feature
Feature: some feature
    Scenario: foo
        When I have a normal scenario
        Then I expect nothing special
    """

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1, tag_expr=tagexpressions.parse('not some_feature'))
            feature = parser.parse()

            feature.should.be.none

    def test_parse_feature_with_ignored_scenario(self):
        """
            Test parsing a feature which has an ignored scenario by a given tag expression
        """
        feature = """
Feature: some feature
    @good_case
    Scenario: foo
        When I have a normal scenario
        Then I expect nothing special

    @bad_case
    Scenario: bad case
        Given I have the number 1
        When I add 3 to my number
        Then I expect my number not to be 4

    @good_case
    Scenario: foo second it
        When I have a normal scenario
        Then I expect nothing special

    """

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1, tag_expr=tagexpressions.parse('not bad_case'))
            feature = parser.parse()

            feature.scenarios.should.have.length_of(2)

    def test_parse_scenario_tag_inheritance(self):
        """
            Test parsing a Scenario which inherits tags
        """
        feature = """
@foo
Feature: some feature
    @good_case
    Scenario: foo
        When I have a normal scenario
        Then I expect nothing special

    @bad_case
    Scenario: bad case
        Given I have the number 1
        When I add 3 to my number
        Then I expect my number not to be 4

    @good_case
    Scenario: foo second it
        When I have a normal scenario
        Then I expect nothing special

    """

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()

            # expect all three Scenarios parsed
            parser = FeatureParser(core, featurefile.name, 1, tag_expr=tagexpressions.parse('foo'))
            feature = parser.parse()
            feature.scenarios.should.have.length_of(3)

            # expect only good case Scenarios parsed
            parser = FeatureParser(core, featurefile.name, 1, tag_expr=tagexpressions.parse('good_case'))
            feature = parser.parse()
            feature.scenarios.should.have.length_of(2)

            # expect only bad case Scenarios parsed
            parser = FeatureParser(core, featurefile.name, 1, tag_expr=tagexpressions.parse('bad_case'))
            feature = parser.parse()
            feature.scenarios.should.have.length_of(1)

    def test_parse_feature_with_syntax_error(self):
        """
            Test parsing a feature which has syntax errors
        """
        feature = """
Bllllla: fooo
    @good_case
    Scenario: foo
        When I have a normal scenario
        Then I expect nothing special"""

        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            parser.parse.when.called.should.throw(FeatureFileSyntaxError)


    def test_parse_simple_background(self):
        """
        Test parsing a feature which has a simple background
        """
        feature = """
Feature: some feature
    Background: Some context
        Given I have a proper setup
        And I have a connection established

    Scenario: foo
        When I have a normal scenario
        Then I expect nothing special

    Scenario: foo second it
        When I have a normal scenario
        Then I expect nothing special
"""
        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            feature = parser.parse()

            feature.background.shouldnt.be.none
            feature.scenarios.should.have.length_of(2)

            feature.background.all_steps.should.have.length_of(2)
            feature.background.all_steps[0].sentence.should.be.equal("Given I have a proper setup")
            feature.background.all_steps[1].sentence.should.be.equal("And I have a connection established")


    def test_parse_simple_background_with_subsequent_scenario_outline(self):
        """
        Test parsing a feature which has a simple background with a subsequent ScenarioOutline
        """
        feature = """
Feature: some feature
    Background: Some context
        Given I have a proper setup
        And I have a connection established

    Scenario Outline: foo
        When I have a normal scenario
        Then I expect nothing special

    Examples:
        | foo | bar |
        | 1   | 2   |

    Scenario: foo second it
        When I have a normal scenario
        Then I expect nothing special
"""
        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            feature = parser.parse()

            feature.background.shouldnt.be.none
            feature.scenarios.should.have.length_of(2)
            feature.scenarios[0].should.be.a(ScenarioOutline)


    def test_parse_simple_background_with_subsequent_scenario_loop(self):
        """
        Test parsing a feature which has a simple background with a subsequent ScenarioLoop
        """
        feature = """
Feature: some feature
    Background: Some context
        Given I have a proper setup
        And I have a connection established

    Scenario Loop 10: foo
        When I have a normal scenario
        Then I expect nothing special

    Scenario: foo second it
        When I have a normal scenario
        Then I expect nothing special
"""
        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            feature = parser.parse()

            feature.background.shouldnt.be.none
            feature.scenarios.should.have.length_of(2)
            feature.scenarios[0].should.be.a(ScenarioLoop)


    def test_parse_simple_background_with_subsequent_tag(self):
        """
        Test parsing a feature which has a simple background with a subsequent tag
        """
        feature = """
Feature: some feature
    Background: Some context
        Given I have a proper setup
        And I have a connection established

    @constant(foo: bar)
    @foo
    Scenario: foo
        When I have a normal scenario
        Then I expect nothing special
"""
        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            feature = parser.parse()

            feature.background.shouldnt.be.none
            feature.scenarios.should.have.length_of(1)
            feature.scenarios[0].constants.should.have.length_of(1)
            feature.scenarios[0].tags.should.have.length_of(2)


    def test_parse_misplaced_background(self):
        """
        Test parsing a feature which has a misplaced background
        """
        feature = """
Feature: some feature
    Scenario: foo
        When I have a normal scenario
        Then I expect nothing special

    Background: Some context
        Given I have a proper setup
        And I have a connection established
"""
        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            parser.parse.when.called.should.throw(FeatureFileSyntaxError, "The Background block must be placed before any Scenario block")


    def test_parse_multiple_backgrounds(self):
        """
        Test parsing a feature which has a multiple backgrounds
        """
        feature = """
Feature: some feature
    Background: Some context
        Given I have a proper setup
        And I have a connection established

    Background: Some context 1
        Given I have a proper setup
        And I have a connection established

    Scenario: foo
        When I have a normal scenario
        Then I expect nothing special
"""
        with NamedTemporaryFile("w+") as featurefile:
            featurefile.write(feature)
            featurefile.flush()

            core = Mock()
            parser = FeatureParser(core, featurefile.name, 1)
            parser.parse.when.called.should.throw(FeatureFileSyntaxError, "The Background block may only appear once in a Feature")



# -*- coding: utf-8 -*-

import os
import re
import tempfile

from tests.base import *

import radish.testing.matches as matches


class MatchesTestCase(RadishTestCase):
    """
    Unit tests for testing.matches module.
    """
    def test_unreasonable_min_coverage(self):
        """
        Test unreasonable minimum test coverage
        """
        matches.test_step_matches_configs.when.called_with(None, None, 101).should.return_value(3)

    def test_no_steps_found(self):
        """
        Test if basedir does not contain any steps to test against
        """
        with patch('radish.testing.matches.load_modules'):
            matches.test_step_matches_configs.when.called_with(None, None).should.return_value(4)


    def test_empty_matches_config(self):
        """
        Test empty matches config file
        """
        # create temporary file
        fd, tmpfile = tempfile.mkstemp()
        os.close(fd)

        with patch('radish.testing.matches.load_modules'), patch('radish.testing.matches.StepRegistry.steps') as steps_mock:
            steps_mock.return_value = [1, 2]
            matches.test_step_matches_configs.when.called_with([tmpfile], None).should.return_value(5)

        # delete temporary file
        os.remove(tmpfile)

    def test_step_matches_invalid_match_config(self):
        """
        Test match config without a sentence and a should_match attribute
        """
        # test config with missing should_match function attribute
        config = [{
            'sentence': None
        }]

        matches.test_step_matches.when.called_with(config, None).should.throw(ValueError,
            'You have to provide a sentence and the function name which should be matched (should_match)')  # pylint: disable=bad-continuation


        # test config with missing sentence attribute
        config = [{
            'should_match': None
        }]

        matches.test_step_matches.when.called_with(config, None).should.throw(ValueError,
            'You have to provide a sentence and the function name which should be matched (should_match)')  # pylint: disable=bad-continuation


        # test empty config
        config = [{}]

        matches.test_step_matches.when.called_with(config, None).should.throw(ValueError,
            'You have to provide a sentence and the function name which should be matched (should_match)')  # pylint: disable=bad-continuation

    def test_sentence_no_step_match(self):
        """
        Test if sentence does not match any step pattern
        """
        steps = {}
        config = [{
            'sentence': 'foo', 'should_match': 'bar'
        }]

        with patch('sys.stdout', new=StringIO()) as out:
            matches.test_step_matches.when.called_with(config, steps).should.return_value((1, 0))
            out.seek(0)
            out.read().should.contain('Expected sentence didn\'t match any step implemention')

    def test_sentence_match_wrong_step(self):
        """
        Test if sentence matched wrong step
        """
        def foo():
            "Test step func"
            pass


        steps = {
            re.compile('foo'): foo
        }
        config = [{
            'sentence': 'foo', 'should_match': 'bar'
        }]

        with patch('sys.stdout', new=StringIO()) as out:
            matches.test_step_matches.when.called_with(config, steps).should.return_value((1, 0))
            out.seek(0)
            out.read().should.contain('Expected sentence matched foo instead of bar')

    def test_sentence_argument_errors(self):
        """
        Test if sentence arguments do not match
        """
        def foo(step, foo, bar):
            "Test step func"
            pass


        steps = {
            re.compile(r'What (.*?) can (.*)'): foo
        }
        config = [{
            'sentence': 'What FOO can BAR', 'should_match': 'foo',
            'with-arguments': [
                {'foo': 'foooooooo'},
                {'bar': 'baaaaaaar'}
            ]
        }]

        with patch('sys.stdout', new=StringIO()) as out:
            matches.test_step_matches.when.called_with(config, steps).should.return_value((1, 0))
            out.seek(0)
            output = out.read()
            output.should.contain('Expected argument "foo" with value "foooooooo" does not match value "FOO"')
            output.should.contain('Expected argument "bar" with value "baaaaaaar" does not match value "BAR"')

    def test_sentence_step_arguments_no_corresponding(self):
        """
        Test sentence step arguments without corresponding actual argument
        """
        expected_arguments = {
            'foo': 'fooo'
        }

        actual_arguments = {
            'FOO': None
        }

        matches.check_step_arguments.when.called_with(expected_arguments, actual_arguments).should.return_value([
            'Expected argument "foo" is not in matched arguments [\'FOO\']'])

    def test_sentence_step_arguments_type_mismatch(self):
        """
        Test sentence step arguments with type mismatch
        """
        expected_arguments = {
            'foo': 'fooo'
        }

        actual_arguments = {
            'foo': 42
        }

        matches.check_step_arguments.when.called_with(expected_arguments, actual_arguments).should.return_value([
            'Expected argument "foo" is of type "int" instead "str"'])

    def test_sentence_step_arguments_value_mismatch(self):
        """
        Test sentence step arguments with value mismatch
        """
        expected_arguments = {
            'foo': 'fooo'
        }

        actual_arguments = {
            'foo': 'foo'
        }

        matches.check_step_arguments.when.called_with(expected_arguments, actual_arguments).should.return_value([
            'Expected argument "foo" with value "fooo" does not match value "foo"'])

    def test_sentence_step_arguments_match(self):
        """
        Test sentence step arguments match
        """
        expected_arguments = {
            'foo': 'foo',
            'bar': 'bar'
        }

        actual_arguments = {
            'foo': 'foo',
            'bar': 'bar'
        }

        matches.check_step_arguments.when.called_with(expected_arguments, actual_arguments).should.return_value([])



# -*- coding: utf-8 -*-

import threading
import re

from tests.base import *

from radish.hookregistry import HookRegistry
from radish.feature import Feature
from radish.scenario import Scenario
from radish.stepmodel import Step
from radish.main import setup_config
import radish.utils as utils

# load extension
from radish.extensions.console_writer import ConsoleWriter
ConsoleWriter()

class ConsoleWriterTestCase(RadishTestCase):
    """
        Tests for the runner class
    """
    @classmethod
    def setUpClass(cls):
        """
            Setup config object
        """
        setup_config({
            "--early_exit": False,
            "--debug-steps": False,
            "--debug-after-failure": False,
            "--inspect-after-failure": False,
            "--bdd-xml": False,
            "--no-ansi": False,
            "--no-line-jump": False,
            "--write-steps-once": False,
            "--with-traceback": False,
            "--marker": False,
            "--write-ids": False
        })

    def tearDown(self):
        """
            Overwrite because HookRegistry should not be reset
        """
        pass

    @classmethod
    def tearDownClass(cls):
        """
            Reset hook registry
        """
        HookRegistry().reset()

    def test_before_each_feature(self):
        """
            Test before.each_feature from extension console_writer
        """
        HookRegistry()
        data = threading.local()
        data.console = None

        def patched_write(text):
            text = re.sub(r"\x1b[^m]*m", "", text)
            data.console = text

        feature = Feature(1, "Feature", "Some feature", "somefile.feature", 1)

        with patch("radish.extensions.console_writer.write", side_effect=patched_write):
            HookRegistry().call("before", "each_feature", feature)

            data.console.should.be.equal("Feature: Some feature  # somefile.feature")

        feature.description.append("This is some description")
        feature.description.append("Because I want to test it")

        with patch("radish.extensions.console_writer.write", side_effect=patched_write):
            HookRegistry().call("before", "each_feature", feature)

            data.console.should.be.equal("""Feature: Some feature  # somefile.feature
    This is some description
    Because I want to test it""")

    def test_before_each_scenario(self):
        """
            Test before.each_scenario from extension console_writer
        """
        data = threading.local()
        data.console = None

        def patched_write(text):
            text = re.sub(r"\x1b[^m]*m", "", text)
            data.console = text

        scenario = Scenario(1, "Scenario", "Some scenario", "somefile.feature", 2, None)
        scenario.parent = Mock(spec=Feature)
        scenario.parent.id = 1

        with patch("radish.extensions.console_writer.write", side_effect=patched_write):
            HookRegistry().call("before", "each_scenario", scenario)

            data.console.should.be.equal("\n    Scenario: Some scenario")

    def test_before_each_step(self):
        """
            Test before.each_step from extension console_writer
        """
        data = threading.local()
        data.console = None

        def patched_write(text):
            text = re.sub(r"\x1b[^m]*m", "", text)
            data.console = text

        scenario_mock = Mock()
        scenario_mock.parent = None
        step = Step(1, "I test the console writer", "somefile.feature", 3, scenario_mock, False)
        step.parent = Mock()
        step.parent.id = 1
        step.parent.parent = Mock(spec=Feature)
        step.parent.parent.id = 1

        with patch("radish.extensions.console_writer.write", side_effect=patched_write):
            HookRegistry().call("before", "each_step", step)

            data.console.should.be.equal("\r        I test the console writer")

    def test_after_each_step(self):
        """
            Test after.each_step from extension console_writer
        """
        data = threading.local()
        data.console = None

        def patched_write(text):
            text = re.sub(r"\x1b[^m]*m", "", text)
            data.console = text

        scenario_mock = Mock()
        scenario_mock.parent = None
        step = Step(1, "I test the console writer", "somefile.feature", 3, scenario_mock, False)
        step.parent = Mock()
        step.parent.id = 1
        step.parent.parent = Mock(spec=Feature)
        step.parent.parent.id = 1

        with patch("radish.extensions.console_writer.write", side_effect=patched_write):
            HookRegistry().call("after", "each_step", step)

            data.console.should.be.equal("\rI test the console writer")

    def test_after_each_step_failed(self):
        """
            Test after.each_step from extension console_writer with failed step
        """
        data = threading.local()
        data.console = None

        def patched_write(text):
            text = re.sub(r"\x1b[^m]*m", "", text)
            data.console = text

        scenario_mock = Mock()
        scenario_mock.parent = None
        step = Step(1, "I test the console writer", "somefile.feature", 3, scenario_mock, False)
        step.parent = MagicMock()
        step.parent.id = 1
        step.parent.parent = Mock(spec=Feature)
        step.parent.parent.id = 1
        step.state = step.State.FAILED
        try:
            assert False, "Some assertion happend"
        except AssertionError as e:
            step.failure = utils.Failure(e)

        with patch("radish.extensions.console_writer.write", side_effect=patched_write):
            HookRegistry().call("after", "each_step", step)

            data.console.should.be.equal("""\rI test the console writer
          AssertionError: Some assertion happend""")



# -*- coding: utf-8 -*-

from tests.base import *

import os

import radish.utils as utils


class UtilsTestCase(RadishTestCase):
    """
        Tests for the utils module
    """
    def test_expandpath(self):
        """
            Test the expandpath utils method
        """
        os.environ["SOMEVAR"] = "foobar"
        utils.expandpath.when.called_with("/some/path/with/$SOMEVAR").should.return_value("/some/path/with/foobar")



# -*- coding: utf-8 -*-

from tests.base import *

from radish.scenario import Scenario
from radish.background import Background
from radish.stepmodel import Step
from radish.model import Tag


class ScenarioTestCase(RadishTestCase):
    """
        Tests for the scenario model class
    """
    def test_scenario_state(self):
        """
            Test scenario's state
        """
        step_1 = Mock(state=Step.State.UNTESTED)
        step_2 = Mock(state=Step.State.UNTESTED)
        step_3 = Mock(state=Step.State.UNTESTED)

        scenario = Scenario(1, "Scenario", "Some scenario", None, None, None)
        scenario.steps.extend([step_1, step_2, step_3])

        scenario.state.should.be.equal(Step.State.UNTESTED)

        step_1.state = Step.State.SKIPPED
        scenario.state.should.be.equal(Step.State.SKIPPED)

        step_1.state = Step.State.FAILED
        scenario.state.should.be.equal(Step.State.FAILED)

        step_2.state = Step.State.PASSED
        scenario.state.should.be.equal(Step.State.FAILED)

        step_3.state = Step.State.PASSED
        scenario.state.should.be.equal(Step.State.FAILED)

        step_1.state = Step.State.PASSED
        scenario.state.should.be.equal(Step.State.PASSED)

    def test_scenario_constants(self):
        """
            Test scenario's constants
        """
        feature = Mock(constants=[("feature_var_1", "1"), ("feature_var_2", "2")])

        scenario = Scenario(1, "Scenario", "Some scenario", None, None, feature)
        scenario.context.constants.extend([("scenario_var_1", "3"), ("scenario_var_2", "4")])

        scenario.constants.should.have.length_of(4)
        scenario.constants[0].should.be.equal(("scenario_var_1", "3"))
        scenario.constants[1].should.be.equal(("scenario_var_2", "4"))
        scenario.constants[2].should.be.equal(("feature_var_1", "1"))
        scenario.constants[3].should.be.equal(("feature_var_2", "2"))

    def test_scenario_all_steps(self):
        """
        Test getting all steps from scenario
        """
        step_1 = Mock(state=Step.State.UNTESTED)
        step_2 = Mock(state=Step.State.UNTESTED)
        step_3 = Mock(state=Step.State.UNTESTED)

        scenario = Scenario(1, "Scenario", "Some scenario", None, None, None)
        scenario.steps.extend([step_1, step_2, step_3])

        scenario.all_steps.should.have.length_of(3)
        scenario.all_steps[0].should.be.equal(step_1)
        scenario.all_steps[1].should.be.equal(step_2)
        scenario.all_steps[2].should.be.equal(step_3)

        precond_step_1 = Mock(state=Step.State.UNTESTED)
        precond_step_2 = Mock(state=Step.State.UNTESTED)
        scenario_precond = Scenario(2, "Scenario", "Some precond scenario", None, None, None)
        scenario_precond.steps.extend([precond_step_1, precond_step_2])

        scenario.preconditions.append(scenario_precond)

        scenario.all_steps.should.have.length_of(5)
        scenario.all_steps[0].should.be.equal(precond_step_1)
        scenario.all_steps[1].should.be.equal(precond_step_2)
        scenario.all_steps[2].should.be.equal(step_1)
        scenario.all_steps[3].should.be.equal(step_2)
        scenario.all_steps[4].should.be.equal(step_3)

    def test_scenario_get_failed_step(self):
        """
            Test getting first failed step from scenario
        """
        step_1 = Mock(state=Step.State.UNTESTED)
        step_2 = Mock(state=Step.State.UNTESTED)
        step_3 = Mock(state=Step.State.UNTESTED)

        scenario = Scenario(1, "Scenario", "Some scenario", None, None, None)
        scenario.steps.extend([step_1, step_2, step_3])

        scenario.failed_step.should.be.equal(None)

        step_1.state = Step.State.FAILED
        scenario.failed_step.should.be.equal(step_1)

        step_1.state = Step.State.PASSED
        scenario.failed_step.should.be.equal(None)

        step_2.state = Step.State.FAILED
        scenario.failed_step.should.be.equal(step_2)

    def test_scenario_has_to_run(self):
        """
            Test scenario's has to run functionality
        """
        feature = Mock()

        s = Scenario(1, "Scenario", "Some scenario", None, None, feature)
        s.absolute_id = 1
        s.has_to_run.when.called_with(None).should.return_value(True)

        s.has_to_run.when.called_with([1]).should.return_value(True)
        s.has_to_run.when.called_with([1, 2]).should.return_value(True)
        s.has_to_run.when.called_with([2]).should.return_value(False)

    def test_scenario_after_parse_hook(self):
        """
            Test scenario after parse hook
        """
        step_1 = Mock(state=Step.State.UNTESTED, id=1)
        step_2 = Mock(state=Step.State.UNTESTED, id=2)
        step_3 = Mock(state=Step.State.UNTESTED, id=3)
        scenario = Scenario(1, "Scenario", "Some scenario", None, None, None)
        scenario.steps.extend([step_1, step_2, step_3])

        step_1.parent = scenario
        step_2.parent = scenario
        step_3.parent = scenario

        scenario.all_steps.should.have.length_of(3)
        scenario.all_steps[0].should.be.equal(step_1)
        scenario.all_steps[1].should.be.equal(step_2)
        scenario.all_steps[2].should.be.equal(step_3)

        scenario_precond = Scenario(2, "Scenario", "Some precond scenario", None, None, None)
        precond_step_1 = Mock(state=Step.State.UNTESTED, id=1)
        precond_step_2 = Mock(state=Step.State.UNTESTED, id=2)
        scenario_precond.steps.extend([precond_step_1, precond_step_2])

        precond_step_1.parent = scenario_precond
        precond_step_2.parent = scenario_precond

        scenario.preconditions.append(scenario_precond)

        # check before 'after_parse': step id and parent should be wrong
        scenario.all_steps[0].id.should.be.equal(1)
        scenario.all_steps[1].id.should.be.equal(2)
        scenario.all_steps[2].id.should.be.equal(1)
        scenario.all_steps[3].id.should.be.equal(2)
        scenario.all_steps[4].id.should.be.equal(3)
        scenario.all_steps[0].parent.should.be.equal(scenario_precond)
        scenario.all_steps[1].parent.should.be.equal(scenario_precond)
        scenario.all_steps[2].parent.should.be.equal(scenario)
        scenario.all_steps[3].parent.should.be.equal(scenario)
        scenario.all_steps[4].parent.should.be.equal(scenario)

        scenario.after_parse()

        scenario.all_steps[0].id.should.be.equal(1)
        scenario.all_steps[1].id.should.be.equal(2)
        scenario.all_steps[2].id.should.be.equal(3)
        scenario.all_steps[3].id.should.be.equal(4)
        scenario.all_steps[4].id.should.be.equal(5)
        scenario.all_steps[0].parent.should.be.equal(scenario)
        scenario.all_steps[1].parent.should.be.equal(scenario)
        scenario.all_steps[2].parent.should.be.equal(scenario)
        scenario.all_steps[3].parent.should.be.equal(scenario)
        scenario.all_steps[4].parent.should.be.equal(scenario)


    def test_scenario_state_with_background(self):
        """
        Test scenario state with assigned background
        """
        bg_step_1 = Mock(state=Step.State.UNTESTED)
        bg_step_2 = Mock(state=Step.State.UNTESTED)
        bg_step_3 = Mock(state=Step.State.UNTESTED)

        step_1 = Mock(state=Step.State.UNTESTED)
        step_2 = Mock(state=Step.State.UNTESTED)
        step_3 = Mock(state=Step.State.UNTESTED)

        all_steps = [bg_step_1, bg_step_2, bg_step_3, step_1, step_2, step_3]

        # create background and scenario
        background = Background('Background', 'Some Background', None, None, None)
        scenario = Scenario(1, "Scenario", "Some scenario", None, None, None)

        # assign background to scenario
        scenario.background = background

        # assign steps to background and scenario
        background.steps.extend([bg_step_1, bg_step_2, bg_step_3])
        scenario.steps.extend([step_1, step_2, step_3])

        # in the beginning all steps are untested
        scenario.state.should.be.equal(Step.State.UNTESTED)

        # lets set them to passed
        for step in all_steps:
            step.state = Step.State.PASSED

        # the scenario should be passed
        scenario.state.should.be.equal(Step.State.PASSED)

        # let's fail a background step
        bg_step_2.state = Step.State.FAILED
        scenario.state.should.be.equal(Step.State.FAILED)

        # let's skip a background step
        bg_step_2.state = Step.State.SKIPPED
        scenario.state.should.be.equal(Step.State.SKIPPED)


    def test_scenario_all_steps_with_background(self):
        """
        Test scenario all_steps with assigned background
        """
        bg_step_1 = Mock(state=Step.State.UNTESTED)
        bg_step_2 = Mock(state=Step.State.UNTESTED)
        bg_step_3 = Mock(state=Step.State.UNTESTED)

        step_1 = Mock(state=Step.State.UNTESTED)
        step_2 = Mock(state=Step.State.UNTESTED)
        step_3 = Mock(state=Step.State.UNTESTED)

        # create background and scenario
        background = Background('Background', 'Some Background', None, None, None)
        scenario = Scenario(1, "Scenario", "Some scenario", None, None, None)

        # assign background to scenario
        scenario.background = background

        # assign steps to background and scenario
        background.steps.extend([bg_step_1, bg_step_2, bg_step_3])
        scenario.steps.extend([step_1, step_2, step_3])

        # in the beginning all steps are untested
        all_steps = scenario.all_steps
        all_steps.should.have.length_of(6)
        all_steps[0].should.be(bg_step_1)
        all_steps[3].should.be(step_1)


    def test_scenario_failed_step_with_background(self):
        """
        Test scenario failed step with assigned background
        """
        bg_step_1 = Mock(state=Step.State.UNTESTED)
        bg_step_2 = Mock(state=Step.State.UNTESTED)
        bg_step_3 = Mock(state=Step.State.UNTESTED)

        step_1 = Mock(state=Step.State.UNTESTED)
        step_2 = Mock(state=Step.State.UNTESTED)
        step_3 = Mock(state=Step.State.UNTESTED)

        all_steps = [bg_step_1, bg_step_2, bg_step_3, step_1, step_2, step_3]

        # create background and scenario
        background = Background('Background', 'Some Background', None, None, None)
        scenario = Scenario(1, "Scenario", "Some scenario", None, None, None)

        # assign background to scenario
        scenario.background = background

        # assign steps to background and scenario
        background.steps.extend([bg_step_1, bg_step_2, bg_step_3])
        scenario.steps.extend([step_1, step_2, step_3])

        # in the beginning all steps are untested
        scenario.state.should.be.equal(Step.State.UNTESTED)

        # lets set them to passed
        for step in all_steps:
            step.state = Step.State.PASSED

        # the scenario should be passed
        scenario.failed_step.should.be.none

        # let's fail a background step
        bg_step_2.state = Step.State.FAILED
        scenario.failed_step.should.be(bg_step_2)



# -*- coding: utf-8 -*-


from unittest import TestCase


class ConfigurationTestCase(TestCase):
    """
        Tests for the runner class
    """
    def _makeOne(self, arguments):
        """
            return instance of class under test
        """
        from radish.core import Configuration
        return Configuration(arguments)

    def test_argument_name_transformation(self):
        """
            Test argument name transformation
        """

        test_arguments = {
            '--doubledash': '',
            '--with-dash': '',
            '<positional>': '',
            '<positional-with-dash>': '',
        }

        returned_config = self._makeOne(test_arguments)

        self.assertTrue(hasattr(returned_config, 'doubledash'))
        self.assertTrue(hasattr(returned_config, 'with_dash'))
        self.assertTrue(hasattr(returned_config, 'positional'))
        self.assertTrue(hasattr(returned_config, 'positional_with_dash'))

    def test_argument_value(self):
        """
            Test argument value setting
        """

        test_arguments = {
            '--argument': 'test_argument_value',
            '<positional-argument>': 'test_positional_argument_value',
        }

        returned_config = self._makeOne(test_arguments)

        self.assertTrue(returned_config.argument, 'test_argument_value')
        self.assertTrue(returned_config.positional_argument,
                        'test_positional_argument_value')



# -*- coding: utf-8 -*-

from tests.base import *

from radish.feature import Feature
from radish.scenario import Scenario
from radish.scenariooutline import ScenarioOutline
from radish.scenarioloop import ScenarioLoop
from radish.stepmodel import Step
from radish.main import setup_config
from radish.model import Tag


class FeatureTestCase(RadishTestCase):
    """
        Tests for the feature model class
    """
    @classmethod
    def setUpClass(cls):
        """
            Setup config object
        """
        setup_config({
            "--early_exit": False,
            "--debug-steps": False,
            "--debug-after-failure": False,
            "--inspect-after-failure": False,
            "--bdd-xml": False,
            "--no-ansi": False,
            "--no-line-jump": False,
            "--write-steps-once": False,
            "--scenarios": None,
            "--shuffle": False,
            "--write-ids": False,
            "--tags": None,
            "--expand": True,
        })

    def test_feature_all_scenarios(self):
        """
            Test getting all scenario from feature
        """
        feature = Feature(1, "Feature", "Some feature", None, None)

        scenario_1 = Mock(spec=Scenario)
        scenario_outline_1_example_1 = Mock(spec=Scenario)
        scenario_outline_1_example_2 = Mock(spec=Scenario)
        scenario_outline_1 = Mock(spec=ScenarioOutline, scenarios=[scenario_outline_1_example_1, scenario_outline_1_example_2])
        scenario_loop_1_example_1 = Mock(spec=Scenario)
        scenario_loop_1_example_2 = Mock(spec=Scenario)
        scenario_loop_1 = Mock(spec=ScenarioLoop, scenarios=[scenario_loop_1_example_1, scenario_loop_1_example_2])
        scenario_2 = Mock(spec=Scenario)

        feature.scenarios.extend([scenario_1, scenario_outline_1, scenario_loop_1, scenario_2])

        feature.all_scenarios.should.have.length_of(8)
        feature.all_scenarios[0].should.be.equal(scenario_1)
        feature.all_scenarios[1].should.be.equal(scenario_outline_1)
        feature.all_scenarios[2].should.be.equal(scenario_outline_1_example_1)
        feature.all_scenarios[3].should.be.equal(scenario_outline_1_example_2)
        feature.all_scenarios[4].should.be.equal(scenario_loop_1)
        feature.all_scenarios[5].should.be.equal(scenario_loop_1_example_1)
        feature.all_scenarios[6].should.be.equal(scenario_loop_1_example_2)
        feature.all_scenarios[7].should.be.equal(scenario_2)

    def test_feature_representations(self):
        """
            Test feature representations
        """
        feature = Feature(1, "Feature", "Some feature", "sometest.feature", 1)
        str(feature).should.be.equal("Feature: Some feature from sometest.feature:1")
        repr(feature).should.be.equal("<Feature: Some feature from sometest.feature:1>")

    def test_feature_scenario_iterator(self):
        """
            Test using a feature as iterator which iterates over its scenarios
        """
        feature = Feature(1, "Feature", "Some feature", None, None)
        feature.scenarios.append(Mock(id=1))
        feature.scenarios.append(Mock(id=2))
        feature.scenarios.append(Mock(id=3))
        feature.scenarios.append(Mock(id=4))

        for expected_scenario_id, scenario in enumerate(feature, start=1):
            scenario.id.should.be.equal(expected_scenario_id)

    def test_feature_get_scenario_by_sentence(self):
        """
            Test getting scenario by sentence from feature
        """
        feature = Feature(1, "Feature", "Some feature", None, None)
        scenario_1 = Mock(sentence="First scenario")
        scenario_2 = Mock(sentence="Second scenario")
        scenario_3 = Mock(sentence="Third scenario")
        feature.scenarios.extend([scenario_1, scenario_2, scenario_3])

        feature["First scenario"].should.be.equal(scenario_1)
        feature["Second scenario"].should.be.equal(scenario_2)
        feature["Third scenario"].should.be.equal(scenario_3)

    def test_feature_state(self):
        """
            Test getting the state of a feature
        """
        feature = Feature(1, "Feature", "Some feature", None, None)
        scenario_1 = Mock(state=Step.State.UNTESTED)
        scenario_2 = Mock(state=Step.State.UNTESTED)
        scenario_outline = Mock(spec=ScenarioOutline, scenarios=[])
        scenario_loop = Mock(spec=ScenarioLoop, scenarios=[])
        scenario_3 = Mock(state=Step.State.UNTESTED)
        feature.scenarios.extend([scenario_1, scenario_2, scenario_outline, scenario_loop, scenario_3])

        feature.state.should.be.equal(Step.State.UNTESTED)

        scenario_1.state = Step.State.SKIPPED
        feature.state.should.be.equal(Step.State.SKIPPED)

        scenario_1.state = Step.State.FAILED
        feature.state.should.be.equal(Step.State.FAILED)

        scenario_2.state = Step.State.PASSED
        feature.state.should.be.equal(Step.State.FAILED)

        scenario_3.state = Step.State.PASSED
        feature.state.should.be.equal(Step.State.FAILED)

        scenario_1.state = Step.State.PASSED
        feature.state.should.be.equal(Step.State.PASSED)

    def test_feature_has_to_run(self):
        """
            Test feature's has to run functionality
        """
        f = Feature(1, "Feature", "Some feature", None, None)
        f.has_to_run.when.called_with(None).should.return_value(True)

        f.scenarios.append(Mock(absolute_id=1, has_to_run=lambda x: False))
        f.scenarios.append(Mock(absolute_id=2, has_to_run=lambda x: False))
        f.scenarios.append(Mock(absolute_id=3, has_to_run=lambda x: False))

        f.has_to_run.when.called_with([1]).should.return_value(True)
        f.has_to_run.when.called_with([1, 2]).should.return_value(True)
        f.has_to_run.when.called_with([3]).should.return_value(True)
        f.has_to_run.when.called_with([1, 4]).should.return_value(True)
        f.has_to_run.when.called_with([5, 4]).should.return_value(False)
        f.has_to_run.when.called_with([6]).should.return_value(False)



# -*- coding: utf-8 -*-

import sys

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

import sure
from unittest import TestCase
from mock import Mock, patch, MagicMock

from radish.stepregistry import StepRegistry
from radish.hookregistry import HookRegistry


def is_python3(minor=0):
    """
        Check if python is in version 3.
    """
    return sys.version_info >= (3, minor)


class RadishTestCase(TestCase):
    """
        Base class for all radish tests.
    """
    def setUp(self):
        """
            Setup test

            Initialize all singletons
        """
        StepRegistry()
        return True

    def tearDown(self):
        """
            Tear down the test

            Delete all singletons
        """
        StepRegistry().clear()
        HookRegistry().reset()



# -*- coding: utf-8 -*-

import threading

from tests.base import *

from radish.runner import Runner
from radish.feature import Feature
from radish.scenario import Scenario
from radish.stepmodel import Step
from radish.main import setup_config


class RunnerTestCase(RadishTestCase):
    """
        Tests for the runner class
    """
    @classmethod
    def setUpClass(cls):
        """
            Setup config object
        """
        setup_config({
            "--early_exit": False,
            "--debug-steps": False,
            "--debug-after-failure": False,
            "--inspect-after-failure": False,
            "--bdd-xml": False,
            "--no-ansi": False,
            "--no-line-jump": False,
            "--write-steps-once": False,
            "--scenarios": None,
            "--shuffle": False,
            "--write-ids": False,
            "--tags": None,
            "--expand": True,
        })

    def test_running_a_step(self):
        """
            Test running a step
        """
        data = threading.local()
        data.step_was_called = False

        def some_step(step):
            data.step_was_called = True

        step = Step(1, "Some step", "somefile.feature", 3, None, True)
        step.definition_func = some_step
        argument_match_mock = Mock()
        argument_match_mock.evaluate.return_value = (tuple(), {})
        step.argument_match = argument_match_mock

        hook_mock = Mock()
        hook_mock.call.return_value = True
        runner = Runner(hook_mock)
        returncode = runner.run_step(step)
        returncode.should.be.equal(0)
        step.state.should.be.equal(Step.State.PASSED)
        data.step_was_called.should.be.true

    def test_running_a_scenario(self):
        """
            Test running a scenario
        """
        data = threading.local()
        data.step_was_called = False

        def some_step(step):
            data.step_was_called = True

        step = Step(1, "Some step", "somefile.feature", 3, None, True)
        step.definition_func = some_step
        argument_match_mock = Mock()
        argument_match_mock.evaluate.return_value = (tuple(), {})
        step.argument_match = argument_match_mock

        scenario = Scenario(1, 1, "Scenario", "Some scenario", "somefile.feature", 2, None)
        scenario.steps.append(step)

        hook_mock = Mock()
        hook_mock.call.return_value = True
        runner = Runner(hook_mock)
        returncode = runner.run_scenario(scenario)
        returncode.should.be.equal(0)
        step.state.should.be.equal(Step.State.PASSED)
        data.step_was_called.should.be.true

    def test_running_a_feature(self):
        """
            Test running a feature
        """
        data = threading.local()
        data.step_was_called = False

        def some_step(step):
            data.step_was_called = True

        feature = Feature(1, "Feature", "Some feature", "somefile.feature", 1)

        scenario = Scenario(1, 1, "Scenario", "Some scenario", "somefile.feature", 2, feature)
        feature.scenarios.append(scenario)

        step = Step(1, "Some step", "somefile.feature", 3, scenario, True)
        step.definition_func = some_step
        argument_match_mock = Mock()
        argument_match_mock.evaluate.return_value = (tuple(), {})
        step.argument_match = argument_match_mock
        scenario.steps.append(step)

        hook_mock = Mock()
        hook_mock.call.return_value = True
        runner = Runner(hook_mock)
        runner.run_feature(feature)
        step.state.should.be.equal(Step.State.PASSED)
        data.step_was_called.should.be.true

    def test_running_all(self):
        """
            Test running a all features
        """
        data = threading.local()
        data.step_was_called = False

        def some_step(step):
            data.step_was_called = True

        feature = Feature(1, "Feature", "Some feature", "somefile.feature", 1)

        scenario = Scenario(1, 1, "Scenario", "Some scenario", "somefile.feature", 2, feature)
        feature.scenarios.append(scenario)

        step = Step(1, "Some step", "somefile.feature", 3, scenario, True)
        step.definition_func = some_step
        argument_match_mock = Mock()
        argument_match_mock.evaluate.return_value = (tuple(), {})
        step.argument_match = argument_match_mock
        scenario.steps.append(step)

        hook_mock = Mock()
        hook_mock.call.return_value = True
        runner = Runner(hook_mock)
        runner.start([feature], None)
        step.state.should.be.equal(Step.State.PASSED)
        data.step_was_called.should.be.true

    def test_returncode_of_runner(self):
        """
            Test returncode of run functions in Runner
        """
        def some_passed_step(step):
            pass

        def some_failed_step(step):
            raise AttributeError("I expect this error to test the behavior of a failed step")

        feature = Feature(1, "Feature", "Some feature", "somefile.feature", 1)

        scenario = Scenario(1, 1, "Scenario", "Some scenario", "somefile.feature", 2, feature)
        feature.scenarios.append(scenario)

        step1 = Step(1, "Some passed step", "somefile.feature", 3, scenario, True)
        step1.definition_func = some_passed_step
        argument_match_mock = Mock()
        argument_match_mock.evaluate.return_value = (tuple(), {})
        step1.argument_match = argument_match_mock
        scenario.steps.append(step1)

        step2 = Step(2, "Some failed step", "somefile.feature", 4, scenario, True)
        step2.definition_func = some_failed_step
        argument_match_mock = Mock()
        argument_match_mock.evaluate.return_value = (tuple(), {})
        step2.argument_match = argument_match_mock
        scenario.steps.append(step2)

        hook_mock = Mock()
        hook_mock.call.return_value = True
        runner = Runner(hook_mock)
        returncode = runner.run_feature(feature)
        returncode.should.be.equal(1)
        step1.state.should.be.equal(Step.State.PASSED)
        step2.state.should.be.equal(Step.State.FAILED)
        scenario.state.should.be.equal(Step.State.FAILED)
        feature.state.should.be.equal(Step.State.FAILED)



# -*- coding: utf-8 -*-

"""
    This module provides a class to match the feature file steps with the registered steps from the registry
"""

import re
from collections import namedtuple

import parse

from .argexpregistry import ArgExpRegistry
from .exceptions import StepDefinitionNotFoundError, StepPatternError

StepMatch = namedtuple("StepMatch", ["argument_match", "func"])


class RegexStepArguments(object):  # pylint: disable=too-few-public-methods
    """Class to represent the argument groups matched by a regex step pattern"""

    def __init__(self, match):
        self.match = match

    def evaluate(self):
        """Lazy and return evaluate the step group matches"""
        return self.match.groups(), self.match.groupdict()


class ParseStepArguments(object):  # pylint: disable=too-few-public-methods
    """Class to represent the argument groups matched by a parse step pattern"""

    def __init__(self, match):
        self.match = match

    def evaluate(self):
        """Lazy and return evaluate the step group matches"""
        result = self.match.evaluate_result()
        return result.fixed, result.named


def merge_steps(features, steps):
    """
        Merges steps from the given features with the given steps

        :param list features: the features
        :param dict steps: the steps
    """
    # FIXME: fix cycle-import ... Matcher -> ScenarioOutline -> Step -> Matcher
    from .scenariooutline import ScenarioOutline
    for feature in features:
        for scenario in feature.all_scenarios:
            if scenario.background:
                for step in scenario.background.steps:
                    merge_step(step, steps)

            if isinstance(scenario, ScenarioOutline):
                continue  # ScenarioOutline steps do not have to be merged

            for step in scenario.steps:
                merge_step(step, steps)


def merge_step(step, steps):
    """
        Merges a single step with the registered steps

        :param Step step: the step from a feature file to merge
        :param list steps: the registered steps
    """
    match = match_step(step.expanded_sentence, steps)
    if not match or not match.func:
        raise StepDefinitionNotFoundError(step)

    step.definition_func = match.func
    step.argument_match = match.argument_match


def match_step(sentence, steps):
    """
        Tries to find a match from the given sentence with the given steps

        :param string sentence: the step sentence to match
        :param dict steps: the available registered steps

        :returns: the arguments and the func which were matched
        :rtype: tuple
    """
    for pattern, func in steps.items():
        if isinstance(pattern, re._pattern_type):  # pylint: disable=protected-access
            match = pattern.search(sentence)
            argument_match = RegexStepArguments(match)
            if match:
                longest_group = get_longest_group(match.regs)
            else:
                longest_group = 0
        else:
            try:
                compiled = parse.compile(pattern, ArgExpRegistry().expressions)
            except ValueError as e:
                raise StepPatternError(pattern, func.__name__, e)

            match = compiled.search(sentence, evaluate_result=False)
            argument_match = ParseStepArguments(match)
            if match:
                longest_group = get_longest_group(match.match.regs)
            else:
                longest_group = 0

        if match:
            if len(sentence) == longest_group:
                return StepMatch(argument_match=argument_match, func=func)

    return None


def get_longest_group(regs):
    if len(regs) < 1:
        return 0

    longest_group = regs[0][1]

    for reg in regs[1:]:
        candidate = reg[1]
        if candidate > longest_group:
            longest_group = candidate
    return longest_group



# -*- coding: utf-8 -*-

import re
from tests.base import *

from radish.matcher import merge_steps, merge_step, match_step
from radish.feature import Feature
from radish.scenario import Scenario
from radish.stepmodel import Step
from radish.exceptions import StepDefinitionNotFoundError
from radish.main import setup_config


class MatcherTestCase(RadishTestCase):
    """
        Tests the Matcher class
    """

    @classmethod
    def setUpClass(cls):
        """
            Setup config object
        """
        setup_config({
            "--basedir": 'foo',
        })

    def test_match_steps(self):
        """
            Test matching steps from feature files with registered steps
        """
        steps = {re.compile(r"Given I have the number (\d+)"): "some_func",
                 re.compile(r"I add (\d+) to my number"): "some_other_func"}

        match, func = match_step("Given I have the number 5", steps)
        arguments, keyword_arguments = match.evaluate()
        arguments.should.be.equal(("5",))
        keyword_arguments.should.be.equal({})
        func.should.be.equal("some_func")

        match, func = match_step("When I add 2 to my number", steps)
        arguments, keyword_arguments = match.evaluate()
        arguments.should.be.equal(("2",))
        keyword_arguments.should.be.equal({})
        func.should.be.equal("some_other_func")

        match = match_step("when I call a non-existing step", steps)
        match.should.be.none  # pylint: disable=pointless-statement

    def test_similar_steps_pattern(self):
        """
            Test matching steps from feature files with similar patterns
        """
        steps = {re.compile(r"Given I have the number (\d+)"): "some_func",
                 re.compile(r"Given I have the number (\d+) to add"): "some_other_func"}

        match, func = match_step("Given I have the number 5", steps)
        arguments, keyword_arguments = match.evaluate()
        arguments.should.be.equal(("5",))
        keyword_arguments.should.be.equal({})
        func.should.be.equal("some_func")

        match, func = match_step("Given I have the number 5 to add", steps)
        arguments, keyword_arguments = match.evaluate()
        arguments.should.be.equal(("5",))
        keyword_arguments.should.be.equal({})
        func.should.be.equal("some_other_func")

        match = match_step("Given I have the number", steps)
        match.should.be.none  # pylint: disable=pointless-statement

    def test_similar_steps(self):
        """
            Test matching steps from feature files with similar text
        """
        steps = {"This is a short sentence": "some_func",
                 "This is a short sentence with few more words": "some_other_func"}

        match, func = match_step("This is a short sentence", steps)
        arguments, keyword_arguments = match.evaluate()
        keyword_arguments.should.be.equal({})
        func.should.be.equal("some_func")

        match, func = match_step("This is a short sentence with few more words", steps)
        arguments, keyword_arguments = match.evaluate()
        keyword_arguments.should.be.equal({})
        func.should.be.equal("some_other_func")

        match = match_step("This is not a step", steps)
        match.should.be.none  # pylint: disable=pointless-statement

    def test_similar_steps_with_args(self):
        """
            Test matching steps from feature files with similar text and arguments
        """
        steps = {"This is a short {arg:S}": "some_func",
                 "This is a short {arg:S} with few more words": "some_other_func"}

        match, func = match_step("This is a short sentence", steps)
        arguments, keyword_arguments = match.evaluate()
        assert keyword_arguments["arg"] == "sentence"
        func.should.be.equal("some_func")

        match, func = match_step("This is a short sentence with few more words", steps)
        arguments, keyword_arguments = match.evaluate()
        assert keyword_arguments["arg"] == "sentence"
        func.should.be.equal("some_other_func")

        match = match_step("This is not a step", steps)
        match.should.be.none  # pylint: disable=pointless-statement

    def test_merge_steps(self):
        """
            Test merging steps from feature files with registered steps
        """
        steps = {re.compile(r"Given I have the number (\d+)"): "some_func",
                 re.compile(r"I add (\d+) to my number"): "some_other_func"}

        feature = Feature(1, "Feature", "Some feature", "test.feature", 1)
        scenario = Scenario(1, "Scenario", "Adding numbers", "test.feature", 2, feature)
        scenario.steps.append(Step(1, "Given I have the number 5", "test.feature", 3, scenario, False))
        scenario.steps.append(Step(2, "When I add 2 to my number", "test.feature", 4, scenario, False))
        feature.scenarios.append(scenario)

        merge_steps([feature], steps)

        scenario.steps[0].definition_func.should.be.equal("some_func")
        scenario.steps[0].argument_match.evaluate()[0].should.be.equal(("5",))
        scenario.steps[1].definition_func.should.be.equal("some_other_func")
        scenario.steps[1].argument_match.evaluate()[0].should.be.equal(("2",))

    def test_merge_non_existing_step(self):
        """
            Test merging non existing step
        """
        steps = {re.compile(r"Given I have the number (\d+)"): "some_func",
                 re.compile(r"I add (\d+) to my number"): "some_other_func"}

        feature = Feature(1, "Feature", "Some feature", "test.feature", 1)
        scenario = Scenario(1, "Scenario", "Adding numbers", "test.feature", 2, feature)
        scenario.steps.append(Step(1, "When I call a non-existing step", "test.feature", 3, scenario, False))
        feature.scenarios.append(scenario)

        merge_steps.when.called_with([feature], steps).should.throw(StepDefinitionNotFoundError)



# -*- coding: utf-8 -*-

"""
This module provides functionaliy to test
if some sentences are matched with the expected
step implementations.
"""

import sys
import codecs

import yaml
from colorful import colorful

from radish.loader import load_modules
from radish.matcher import match_step
from radish.stepregistry import StepRegistry
from radish.utils import get_func_arg_names, get_func_location


def test_step_matches_configs(match_config_files, basedir, cover_min_percentage=None, cover_show_missing=False):
    """
    Test if the given match config files matches the actual
    matched step implementations.
    """
    if cover_min_percentage is not None and float(cover_min_percentage) > 100:
        sys.stderr.write(colorful.magenta('You are a little cocky to think you can reach a minimum coverage of {0:.2f}%\n'.format(
            float(cover_min_percentage))))
        return 3

    # load user's custom python files
    load_modules(basedir)
    steps = StepRegistry().steps

    if not steps:
        sys.stderr.write(colorful.magenta('No step implementations found in {0}, thus doesn\'t make sense to continue'.format(
            basedir)))
        return 4

    failed = 0
    passed = 0
    covered_steps = set()

    for match_config_file in match_config_files:
        # load the given match config file
        with codecs.open(match_config_file, "r", "utf-8") as f:
            match_config = yaml.safe_load(f)

        if not match_config:
            print(colorful.magenta('No sentences found in {0} to test against'.format(match_config_file)))
            return 5
        else:
            print(colorful.brown('Testing sentences from {0}:'.format(colorful.bold_brown(match_config_file))))
            failed_sentences, passed_senteces = test_step_matches(match_config, steps)
            failed += failed_sentences
            passed += passed_senteces

            covered_steps = covered_steps.union(x['should_match'] for x in match_config)

        # newline
        sys.stdout.write('\n')

    report = colorful.bold_white('{0} sentences ('.format(failed + passed))
    if passed > 0:
        report += colorful.bold_green('{0} passed'.format(passed))

    if passed > 0 and failed > 0:
        report += colorful.bold_white(', ')

    if failed > 0:
        report += colorful.bold_red('{0} failed'.format(failed))
    report += colorful.bold_white(')')
    print(report)

    step_coverage = 100.0 / len(steps) * len(covered_steps)
    coverage_report = colorful.bold_white('Covered {0} of {1} step implementations'.format(
        len(covered_steps), len(steps)))

    ret = 0 if failed == 0 else 1

    if cover_min_percentage:
        coverage_color = colorful.bold_green if step_coverage >= float(cover_min_percentage) else colorful.bold_red
        coverage_report += colorful.bold_white(' (coverage: ')
        coverage_report += coverage_color('{0:.2f}%'.format(step_coverage))
        if float(cover_min_percentage) > step_coverage:
            coverage_report += colorful.bold_white(', expected a minimum of {0}'.format(
                colorful.bold_green(cover_min_percentage + '%')))
            if failed == 0:
                ret = 2
            # if tests have passed and coverage is too low we fail with exit code 2
        coverage_report += colorful.bold_white(')')

    print(coverage_report)

    if cover_show_missing:
        missing_steps = get_missing_steps(steps, covered_steps)
        if missing_steps:
            missing_step_report = colorful.bold_brown('Missing steps:\n')
            for step in missing_steps:
                missing_step_report += '- {0} at '.format(
                    colorful.cyan(step[0]))
                missing_step_report += colorful.cyan(step[1]) + '\n'
            sys.stdout.write(missing_step_report)

    return ret


def test_step_matches(match_config, steps):
    """
    Test if the given match config matches the actual
    matched step implementations.
    """
    failed = 0
    passed = 0

    for item in match_config:
        if 'sentence' not in item or 'should_match' not in item:
            raise ValueError('You have to provide a sentence and the function name which should be matched (should_match)')

        sentence = item['sentence']
        expected_step = item['should_match']

        sys.stdout.write('{0} STEP "{1}" SHOULD MATCH {2}    '.format(
            colorful.brown('>>'), colorful.cyan(sentence), colorful.cyan(expected_step)))

        result = match_step(item['sentence'], steps)
        if not result:
            output_failure(None, ['Expected sentence didn\'t match any step implemention'])
            failed += 1
            continue

        if expected_step != result.func.__name__:
            output_failure(result.func, ['Expected sentence matched {0} instead of {1}'.format(result.func.__name__, expected_step)])
            failed += 1
            continue


        expected_arguments = item.get('with-arguments')

        if expected_arguments:
            arguments = merge_step_args(result)
            expected_arguments = {k: v for expected_arguments in expected_arguments for k, v in expected_arguments.items()}
            argument_errors = check_step_arguments(expected_arguments, arguments)
            if argument_errors:
                output_failure(result.func, argument_errors)
                failed += 1
                continue

        # check if arguments match
        print(colorful.bold_green('✔'))
        passed += 1

    return failed, passed


def output_failure(step_func, errors):
    """
    Write the given errors to stdout.
    """
    sys.stdout.write(colorful.bold_red('✘'))
    if step_func is not None:
        sys.stdout.write(colorful.red(' (at {0})'.format(get_func_location(step_func))))

    sys.stdout.write('\n')

    for error in errors:
        print(colorful.red('  - {0}'.format(error)))


def check_step_arguments(expected_arguments, arguments):
    """
    Check if the given expected arguments
    match the actual arguments
    """
    errors = []
    for arg_name, arg_value in expected_arguments.items():
        if arg_name not in arguments:
            errors.append('Expected argument "{0}" is not in matched arguments {1}'.format(
                arg_name, list(arguments.keys())))
            continue

        # check if argument value is a dict, if yes we'll do thorough comparison
        if isinstance(arg_value, dict):
            _type = arg_value['type']
            value = arg_value['value']
        else:
            _type = type(arg_value).__name__
            value = arg_value

        if type(arguments[arg_name]).__name__ != _type:
            errors.append('Expected argument "{0}" is of type "{1}" instead "{2}"'.format(
                arg_name, type(arguments[arg_name]).__name__, _type))
            continue

        if arguments[arg_name] != value:
            errors.append('Expected argument "{0}" with value "{1}" does not match value "{2}"'.format(
                arg_name, value, arguments[arg_name]))
    return errors


def merge_step_args(step_func):
    """
    Merges the arguments and keyword arguments
    of the given step function.
    """
    #: Holds the merged arguments as a dict with the corresponding matched values
    args, kwargs = step_func.argument_match.evaluate()
    step_arg_names = get_func_arg_names(step_func.func)[1:]
    arguments = dict(zip(step_arg_names, args))
    arguments.update(kwargs)
    return arguments


def get_missing_steps(steps, covered_steps):
    """
    Get all steps within ``steps`` which are not
    covered by ``covered_steps``.
    """
    missing_steps = []
    for step_func in steps.values():
        if step_func.__name__ not in covered_steps:
            missing_steps.append((step_func.__name__, get_func_location(step_func)))
    return missing_steps



# -*- coding: utf-8 -*-

"""
    This module provides the Argument-Expression Registry
"""

from singleton import singleton

from .exceptions import RadishError

@singleton()
class ArgExpRegistry(object):
    """
        Registry for all custom argument expressions
    """
    def __init__(self):
        self._expressions = {}

    def register(self, name, func):
        """
            Registers a custom argument expression
        """
        if name in self._expressions:
            raise RadishError("Cannot register argument expression with name {0} because it already exists".format(name))

        self._expressions[name] = func

    @property
    def expressions(self):
        """
            Returns all registered expressions
        """
        return self._expressions


def arg_expr(name, expression):
    """
        Decorator for custom argument expressions
    """
    def _decorator(func):
        """
            Actual decorator
        """
        func.pattern = expression
        ArgExpRegistry().register(name, func)

        return func
    return _decorator
